import { action } from '@ember/object';
import { inject as service } from '@ember/service';
import Component from '@glimmer/component';
import { debounce } from '@ember/runloop';
import { tracked } from '@glimmer/tracking'
import { task } from 'ember-concurrency';
import { getOwner } from '@ember/application';
import zxcvbn from 'zxcvbn';

export default class PasswordResetComponentForm extends Component {
  @service router;
  @service popupAlert;
  @service request;

  @tracked password = null;
  @tracked passwordConfirmation = null;
  @tracked resetPasswordToken = null;
  @tracked passStrength = 0;
  @tracked isInviteAccept;

  get hasPassword() {
    return !!this.password;
  }

  get validPasswordLength() {
    return this.passStrength >= 2;
  }

  get hasResetToken() {
    return !!this.resetPasswordToken;
  }

  get isValid() {
    return this.hasPassword && this.passwordsMatch && this.validPasswordLength && this.hasResetToken;
  }

  get isInvalid() {
    return !this.isValid;
  }

  get passwordsMatch() {
    return !!this.password && this.password == this.passwordConfirmation;
  }

  get errors() {
    let errors = [];
    if (!this.hasPassword) {
      errors.pushObject('Password is required');
    }

    if (!this.validPasswordLength && this.hasPassword) {
      errors.pushObject('Password must be strong');
    }

    if (!this.passwordsMatch) {
      errors.pushObject('Passwords must match');
    }

    return errors;
  }

  get inputPlaceholder() {
    return this.isInviteAccept ? 'Password' : 'New Password';
  }

  get url() {
    return this.isInviteAccept
      ? `${
          getOwner(this).resolveRegistration('config:environment').namespace
        }/users/accept_invite`
      : 'password_set';
  }

  _setPassStrength() {
    let score = this.password ? zxcvbn(this.password).score : 0;
    this.passStrength = score;
    this.args?.setPassStrength(score);
  }

  @task(function* (password, passwordConfirmation) {
    let data = {
      reset_password_token: this.resetPasswordToken,
      password,
      password_confirmation: passwordConfirmation,
    };

    let response = yield this.request.request(
      this.url,
      data,
      this.isInviteAccept
    );

    if (response.ok) {
      this.popupAlert.send(
        this.isInviteAccept
          ? 'Successfully created account! Please sign in.'
          : 'Successfully set new password.',
        'success'
      );
      this.router.transitionTo('login');
    } else {
      this.router.transitionTo('forgot-password');
      this.popupAlert.send(
        'Your password set either expired or something went wrong. Please try again and use the most recent email link.',
        'alert'
      );
    }
  })
  resetPassword;

  @action
  _passwordDidChange({ target }) {
    this.password = target.value;
    debounce(this, this._setPassStrength, 200);
    this.args?.passwordDidChange(target.value);

  }

  @action
  _passwordConfirmationDidChange({ target }) {
    this.passwordConfirmation = target.value;
    this.args?.passwordConfirmationDidChange(target.value);
  }
}
