import { getOwner } from '@ember/application';
import { assert } from '@ember/debug';
import { inject as service } from '@ember/service';
import SimpleAuthSessionService from 'ember-simple-auth/services/session';
// eslint-disable-next-line ember/no-computed-properties-in-native-classes
import { or, equal, not, readOnly } from '@ember/object/computed';

import { tracked } from '@glimmer/tracking';
import { task } from 'ember-concurrency-decorators';

const { log } = console;

export default class AdminSessionService extends SimpleAuthSessionService {
  @service pushNotification;
  @service popupAlert;
  @service store;
  @service router;
  @service socket;

  @tracked user = null;
  @tracked brokers = null; // array of broker objects with location and building_id keys
  @tracked jwt = null; // User specific broker access jwt

  @readOnly('data.authenticated.current_user.user_role')
  userRole;

  @readOnly('data.authenticated.current_user.building_id')
  buildingId;

  @equal('userRole', 'Manager')
  isManager;

  @equal('userRole', 'BuildingAdmin')
  isBuildingAdmin;

  @equal('userRole', 'OrgManager')
  isOrgManager;

  @equal('userRole', 'Super')
  isSuper;

  @or('isOrgManager', 'isSuper')
  isOrgAdmin;

  @not('isManager')
  notManager;

  @not('isBuildingAdmin')
  notBuildingAdmin;

  @not('isOrgAdmin')
  notOrgAdmin;

  @not('isSuper')
  notSuper;

  @or('isManager', 'isBuildingAdmin', 'isOrgManager', 'isSuper')
  hasRole;

  @or('isManager', 'isBuildingAdmin', 'isOrgManager', 'isSuper')
  isAdminAppRole;

  @or('isSuper', 'isOrgManager', 'isBuildingAdmin')
  hasEdit;

  async load() {
    await this._load.perform();

    if (!getOwner(this).resolveRegistration('config:environment').isTest) {
      this._subscribe.perform();
      this._subscribeWS();
    }
  }

  get authenticatedHeaders() {
    if (!this.isAuthenticated) return {};
    const { accessToken, client, expiry, tokenType, uid } =
      this.data.authenticated;
    return {
      'access-token': accessToken,
      'token-type': tokenType,
      client: client,
      expiry: expiry,
      uid: uid,
    };
  }

  @task
  _subscribe = function *() {
    const subscription = yield this.pushNotification.subscribe.perform();

    if (subscription) {
      log(JSON.stringify(subscription));
    }
  }

  _subscribeWS() {
    const subTo = { channel: 'AlertChannel', user: this.user.id };
    this.socket.subscribeTo(subTo, {
      connected: () => { console.log("WebSocket connected"); },
      received: (wsMessage) => { 
        const data = JSON.parse(wsMessage);
        this.store.pushPayload(data);
        this.popupAlert.send(data.data.attributes.description, 'alert');
      },
      disconnected: () => { console.log("WebSocket disconnected"); }
    });
  }

  @task
  _load = function *() {
    if (!this.isAuthenticated) {
      return;
    }

    const { current_user, broker_location, jwt } = this.data.authenticated;

    let user = yield this.store.findRecord('user', current_user.id);
    this.user = user;
    this.brokers = broker_location ? JSON.parse(broker_location) : [];
    this.jwt = jwt
  }

  handleAuthentication() {
    this.router.transitionTo('admin.buildings.index');
  }

  _validatePayload() {
    assert(
      'Your session payload is invalid: no current_user was present in the session payload.',
      this.data.authenticated.current_user
    );
  }
}
